==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/saml/SAMLAuthnUtil.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 21, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.saml;

import java.security.cert.X509Certificate;

import org.w3c.dom.Document;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.SAMLUtils;
import eu.unicore.samly2.exceptions.SAMLProtocolException;
import eu.unicore.samly2.exceptions.SAMLRequestException;
import eu.unicore.samly2.proto.AssertionResponse;
import eu.unicore.samly2.proto.AuthnRequest;
import eu.unicore.security.dsig.DSigException;
import eu.unicore.security.dsig.DigSignatureUtil;

import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.server.CryptoDataProvider;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.org.oasis.saml2.assertion.NameIDType;
import xmlbeans.org.oasis.saml2.protocol.AuthnRequestDocument;
import xmlbeans.org.oasis.saml2.protocol.AuthnRequestType;
import xmlbeans.org.oasis.saml2.protocol.NameIDPolicyType;

/**
 * Utility for handling request->response chain in case of Auth protocol.
 * @author K. Benedyczak
 */
public class SAMLAuthnUtil extends SAMLUtil
{
	private AuthnRequestDocument requestDoc;
	private AuthnRequestType request;

	private String requestedFormat;
	private String issuerName;
	private String reqId;
	private String consumerURL;
	
	public SAMLAuthnUtil(AuthnRequestDocument requestDoc, 
			CryptoDataProvider secProvider, Configuration conf, SAMLConfiguration samlConfig)
	{
		super(requestDoc.getAuthnRequest(), secProvider, conf, samlConfig);
		this.requestDoc = requestDoc;
	}
	
	public void parse() throws SAMLProtocolException
	{
		new AuthnRequest(requestDoc);
		
		request = requestDoc.getAuthnRequest();
		
		NameIDPolicyType nameIDPolicy = request.getNameIDPolicy();
		if (nameIDPolicy == null)
			requestedFormat = null;
		else
			requestedFormat = nameIDPolicy.getFormat();
		if (requestedFormat == null)
			requestedFormat = SAMLConstants.NFORMAT_UNSPEC;
		NameIDType issuer = request.getIssuer();
		if (issuer == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
				"SAML Authentication request MUST contain an Issuer element.");
		String format = issuer.getFormat();
		if (format != null && !format.equals(SAMLConstants.NFORMAT_ENTITY))
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
			"SAML Authentication request Issuer MUST be of type " + 
			SAMLConstants.NFORMAT_ENTITY + " (which is a default type).");
		issuerName = issuer.getStringValue();
		if (issuerName == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
					"SAML Authentication request Issuer MUST not be empty.");
		reqId = request.getID();
		if (reqId == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
					"SAML Authentication request MUST contain ID.");
		consumerURL = request.getAssertionConsumerServiceURL();
		if (consumerURL == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
			"This implementation requires that consumerServiceURL is in " +
			"SAML Authentication.");
	}

	public String getReqId()
	{
		return reqId;
	}

	public AuthnRequestDocument getRequestDoc()
	{
		return requestDoc;
	}
	
	public boolean isRequestSigned()
	{
		if (request.getSignature() == null 
				|| request.getSignature().isNil())
			return false;
		else return true;
	}
	
	public boolean isRequestCorrectlySigned() throws DSigException
	{
		if (!isRequestSigned())
			return false;
		DigSignatureUtil sign = new DigSignatureUtil();
		X509Certificate[] issuerC = SAMLUtils.getIssuerFromSignature(
				request.getSignature());
		if (issuerC == null || issuerC.length == 0)
			throw new DSigException("Issuer certificate is not present in signature.");
		return sign.verifyEnvelopedSignature(
			(Document) requestDoc.getDomNode(), issuerC[0].getPublicKey());
	}

	
	public AssertionResponse getOKResponseDocument()
	{
		return new AssertionResponse(getIssuer(), request.getID());
	}
	
	public AssertionResponse getErrorResponse(UVOSException e) 
		throws UVOSFault
	{
		return getErrorResponse(convert2SAMLError(e, null, true));
	}	

	public AssertionResponse getErrorResponse(UVOSException e, String message) 
		throws UVOSFault
	{
		return getErrorResponse(convert2SAMLError(e, message, false));
	}	

	public AssertionResponse getErrorResponse(SAMLProtocolException e)
	{
		String id = null;
		if (request == null)
		{
			if (requestDoc != null && requestDoc.getAuthnRequest() != null
				&& requestDoc.getAuthnRequest().getID() != null)
				id = requestDoc.getAuthnRequest().getID();
		} else
			id = request.getID();
		return new AssertionResponse(getIssuer(), id, e);
	}

	@Override
	public X509Certificate[] getIssuerIdentityFromSignature() throws DSigException
	{
		throw new DSigException("Not implemented for Autn protocol");
	}

	public String getRequestedFormat()
	{
		return requestedFormat;
	}

	public String getIssuerName()
	{
		return issuerName;
	}

	public String getConsumerURL()
	{
		return consumerURL;
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/saml/SAMLAuthnUtil.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 21, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.saml;

import java.security.cert.X509Certificate;
import java.util.Collections;

import org.w3c.dom.Document;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.SAMLUtils;
import eu.unicore.samly2.exceptions.SAMLProtocolException;
import eu.unicore.samly2.exceptions.SAMLRequestException;
import eu.unicore.samly2.proto.AbstractRequest;
import eu.unicore.samly2.proto.AssertionResponse;
import eu.unicore.samly2.proto.AuthnRequest;
import eu.unicore.security.dsig.DSigException;
import eu.unicore.security.dsig.DigSignatureUtil;

import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.server.IssuerIdentityProvider;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.org.oasis.saml2.assertion.NameIDType;
import xmlbeans.org.oasis.saml2.protocol.AuthnRequestDocument;
import xmlbeans.org.oasis.saml2.protocol.AuthnRequestType;
import xmlbeans.org.oasis.saml2.protocol.NameIDPolicyType;

/**
 * Utility for handling request->response chain in case of Auth protocol.
 * @author K. Benedyczak
 */
public class SAMLAuthnUtil extends SAMLUtil
{
	private AuthnRequestDocument requestDoc;
	private AuthnRequestType request;

	private String requestedFormat;
	private String issuerName;
	private String reqId;
	private String consumerURL;
	
	public SAMLAuthnUtil(AuthnRequestDocument requestDoc, 
			IssuerIdentityProvider secProvider, UVOSServerProperties conf, SAMLConfiguration samlConfig)
	{
		super(requestDoc.getAuthnRequest(), secProvider, conf, samlConfig);
		this.requestDoc = requestDoc;
	}
	
	public void parse() throws SAMLProtocolException
	{
		new AuthnRequest(requestDoc);
		
		request = requestDoc.getAuthnRequest();
		
		NameIDPolicyType nameIDPolicy = request.getNameIDPolicy();
		if (nameIDPolicy == null)
			requestedFormat = null;
		else
			requestedFormat = nameIDPolicy.getFormat();
		if (requestedFormat == null)
			requestedFormat = SAMLConstants.NFORMAT_UNSPEC;
		NameIDType issuer = request.getIssuer();
		if (issuer == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
				"SAML Authentication request MUST contain an Issuer element.");
		String format = issuer.getFormat();
		if (format != null && !format.equals(SAMLConstants.NFORMAT_ENTITY))
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
			"SAML Authentication request Issuer MUST be of type " + 
			SAMLConstants.NFORMAT_ENTITY + " (which is a default type).");
		issuerName = issuer.getStringValue();
		if (issuerName == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
					"SAML Authentication request Issuer MUST not be empty.");
		reqId = request.getID();
		if (reqId == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
					"SAML Authentication request MUST contain ID.");
		consumerURL = request.getAssertionConsumerServiceURL();
		if (consumerURL == null)
			throw new SAMLRequestException(SAMLConstants.STATUS2_REQUEST_UNSUPP, 
			"This implementation requires that consumerServiceURL is in " +
			"SAML Authentication.");
	}

	public String getReqId()
	{
		return reqId;
	}

	public AuthnRequestDocument getRequestDoc()
	{
		return requestDoc;
	}
	
	public boolean isRequestSigned()
	{
		if (request.getSignature() == null 
				|| request.getSignature().isNil())
			return false;
		else return true;
	}
	
	public boolean isRequestCorrectlySigned() throws DSigException
	{
		if (!isRequestSigned())
			return false;
		DigSignatureUtil sign = new DigSignatureUtil();
		X509Certificate[] issuerC = SAMLUtils.getIssuerFromSignature(
				request.getSignature());
		if (issuerC == null || issuerC.length == 0)
			throw new DSigException("Issuer certificate is not present in signature.");
		Document doc = (Document) requestDoc.getDomNode(); 
		return sign.verifyEnvelopedSignature(doc,
			Collections.singletonList(doc.getDocumentElement()),
			AbstractRequest.PROTOCOL_ID_QNAME,
			issuerC[0].getPublicKey());
	}

	
	public AssertionResponse getOKResponseDocument()
	{
		return new AssertionResponse(getIssuer(), request.getID());
	}
	
	public AssertionResponse getErrorResponse(UVOSException e) 
		throws UVOSFault
	{
		return getErrorResponse(convert2SAMLError(e, null, true));
	}	

	public AssertionResponse getErrorResponse(UVOSException e, String message) 
		throws UVOSFault
	{
		return getErrorResponse(convert2SAMLError(e, message, false));
	}	

	public AssertionResponse getErrorResponse(SAMLProtocolException e)
	{
		String id = null;
		if (request == null)
		{
			if (requestDoc != null && requestDoc.getAuthnRequest() != null
				&& requestDoc.getAuthnRequest().getID() != null)
				id = requestDoc.getAuthnRequest().getID();
		} else
			id = request.getID();
		return new AssertionResponse(getIssuer(), id, e);
	}

	@Override
	public X509Certificate[] getIssuerIdentityFromSignature() throws DSigException
	{
		throw new DSigException("Not implemented for Autn protocol");
	}

	public String getRequestedFormat()
	{
		return requestedFormat;
	}

	public String getIssuerName()
	{
		return issuerName;
	}

	public String getConsumerURL()
	{
		return consumerURL;
	}
}
